"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const os_1 = require("os");
const path_1 = require("path");
const docker_command_1 = __importDefault(require("../../docker-command"));
const version_1 = __importDefault(require("../../version"));
const status_1 = __importDefault(require("./status"));
class Start extends docker_command_1.default {
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { flags } = this.parse(Start);
            const status = yield status_1.default.run(['--name', flags.name, '--silent', ...this.flagsAsArgs(flags)]);
            if (status === 0 /* STARTED */) {
                this.log('Engine is already started');
                return false;
            }
            if (flags.pull) {
                this.spinner.start(`Pulling version ${flags.version}`);
                yield this.pull(flags.version);
                this.spinner.stop();
            }
            this.spinner.start('Starting Engine');
            const eventPromise = this.waitForEvent(({ Action, Type, from }) => Type === 'container' &&
                Action === 'start' &&
                from === `mesg/engine:${flags.version}`);
            const network = yield this.getOrCreateNetwork({ name: flags.name });
            yield this.createEngineService(network, {
                name: flags.name,
                version: flags.version,
                path: flags.path,
                port: flags.port,
                p2pPort: flags['p2p-port'],
            });
            yield eventPromise;
            yield this.waitForAPI();
            this.spinner.stop();
            return true;
        });
    }
    waitForAPI() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.api.service.list({});
            }
            catch (e) {
                // code 2: Error: 2 UNKNOWN: Stream removed
                // code 14: Error: 14 UNAVAILABLE: failed to connect to all addresses
                if (e.code === 2 || e.code === 14) {
                    return new Promise(resolve => {
                        setTimeout(() => this.waitForAPI().then(resolve), 1000);
                    });
                }
                throw e;
            }
        });
    }
}
exports.default = Start;
Start.description = 'Start the Engine';
Start.flags = Object.assign(Object.assign({}, docker_command_1.default.flags), { version: command_1.flags.string({
        description: 'Version of the Engine to run',
        required: true,
        default: version_1.default.engine
    }), path: command_1.flags.string({
        description: 'Path to the mesg folder',
        default: path_1.join(os_1.homedir(), '.mesg'),
        required: true,
    }), pull: command_1.flags.boolean({
        description: 'Pull the latest image of the given version',
        default: true,
        allowNo: true
    }), 'p2p-port': command_1.flags.integer({
        description: 'Port to use for p2p interaction',
        default: 26656,
        required: true,
    }) });
