"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const compiler_1 = require("@mesg/compiler");
const fs_1 = require("fs");
const path_1 = require("path");
const root_command_1 = __importDefault(require("../../root-command"));
const error_1 = require("../../utils/error");
const compile_1 = __importDefault(require("../service/compile"));
class ProcessCompile extends root_command_1.default {
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { args, flags } = this.parse(ProcessCompile);
            const definition = yield compiler_1.process(fs_1.readFileSync(args.PROCESS_FILE), (instanceObject) => __awaiter(this, void 0, void 0, function* () {
                if (instanceObject.instanceHash) {
                    return instanceObject.instanceHash;
                }
                if (!flags.dev) {
                    throw new Error('"instanceHash" should be present in your process. Use `--dev` to be able to use the "instance" attributes.');
                }
                if (!instanceObject.instance) {
                    throw new Error('"instanceHash" or "instance" not found in the process\'s definition');
                }
                const { service, src, env } = instanceObject.instance;
                if (service) {
                    return this.serviceToInstance(service, env);
                }
                if (src) {
                    return this.sourceToInstance(args.PROCESS_FILE, src, env, flags);
                }
                throw new Error('at least one of the following parameter should be set: "instanceHash", "service" or "src"');
            }), (flags.env || []).reduce((prev, env) => (Object.assign(Object.assign({}, prev), { [env.split('=')[0]]: env.split('=')[1] })), {}));
            this.styledJSON(definition);
            this.spinner.stop();
            return definition;
        });
    }
    sourceToInstance(dir, src, env, flags) {
        return __awaiter(this, void 0, void 0, function* () {
            const directory = path_1.join(path_1.dirname(dir), src);
            const definition = yield compile_1.default.run([fs_1.existsSync(directory) ? directory : src, '--silent', ...this.flagsAsArgs(flags)]);
            const { hash } = yield this.api.service.hash(definition);
            if (!hash)
                throw new Error('invalid hash');
            const { exists } = yield this.api.service.exists({ hash });
            if (!exists) {
                const resp = yield this.api.service.create(definition);
                if (!resp.hash)
                    throw new Error('invalid hash');
                if (resp.hash.toString() !== hash.toString())
                    throw new Error('invalid hash');
            }
            return this.serviceToInstance(hash, env);
        });
    }
    serviceToInstance(sidOrHash, env) {
        return __awaiter(this, void 0, void 0, function* () {
            const { services } = yield this.api.service.list({});
            if (!services)
                throw new Error('no services deployed, please deploy your service first');
            const sidOrHashStr = sidOrHash.toString();
            const match = services.filter(x => x.hash && x.hash.toString() === sidOrHashStr || x.sid && x.sid.toString() === sidOrHashStr);
            if (!match || match.length === 0)
                throw new Error(`cannot find any service with the following: ${sidOrHashStr}`);
            if (match.length > 1)
                throw new Error(`multiple services match the following sid: ${sidOrHashStr}, provide a service's hash instead`);
            const service = match[0];
            if (!service.hash)
                throw new Error('invalid service');
            // get runner
            var runnerHash;
            try {
                const { hash } = yield this.api.runner.create({
                    serviceHash: service.hash,
                    env,
                });
                if (!hash)
                    throw new Error('invalid runner created hash');
                runnerHash = hash;
            }
            catch (e) {
                if (!error_1.IsAlreadyExistsError.match(e))
                    throw e;
                runnerHash = new error_1.IsAlreadyExistsError(e).hash;
            }
            if (!runnerHash)
                throw new Error('invalid runner hash');
            const runner = yield this.api.runner.get({ hash: runnerHash });
            return runner.instanceHash;
        });
    }
}
exports.default = ProcessCompile;
ProcessCompile.description = 'Compile a process';
ProcessCompile.flags = Object.assign(Object.assign({}, root_command_1.default.flags), { dev: command_1.flags.boolean({ description: 'compile the process and automatically deploy and start all the services' }), env: command_1.flags.string({
        description: 'Set environment variables',
        multiple: true,
        helpValue: 'FOO=BAR'
    }) });
ProcessCompile.args = [{
        name: 'PROCESS_FILE',
        description: 'Path of a process file'
    }];
