"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const types_1 = require("@mesg/api/lib/types");
const b58 = __importStar(require("@mesg/api/lib/util/base58"));
const encoder_1 = require("@mesg/api/lib/util/encoder");
const util_1 = require("util");
const root_command_1 = __importDefault(require("../../root-command"));
class ProcessLogs extends root_command_1.default {
    constructor() {
        super(...arguments);
        this.services = {};
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { args } = this.parse(ProcessLogs);
            const process = yield this.api.process.get({
                hash: b58.decode(args.PROCESS_HASH)
            });
            if (!process.hash) {
                throw new Error('invalid process hash');
            }
            if (process.nodes) {
                const instanceHashes = process.nodes
                    .map(({ event, result, task }) => {
                    if (event)
                        return event.instanceHash;
                    if (result)
                        return result.instanceHash;
                    if (task)
                        return task.instanceHash;
                })
                    .filter(x => x && x.length > 0);
                for (const hash of instanceHashes) {
                    if (!hash)
                        continue;
                    const instance = yield this.api.instance.get({ hash });
                    const service = yield this.api.service.get({ hash: instance.serviceHash });
                    this.services[b58.encode(hash)] = service;
                }
            }
            const streams = [];
            const results = this.api.execution.stream({
                filter: {
                    statuses: [
                        types_1.ExecutionStatus.COMPLETED,
                        types_1.ExecutionStatus.FAILED,
                    ],
                }
            });
            streams.push(() => results.cancel());
            results
                .on('data', this.handlerResult(process.hash))
                .on('error', (error) => { this.warn('Result stream error: ' + error.message); });
            return {
                destroy: () => {
                    streams.forEach(s => s());
                }
            };
        });
    }
    handlerResult(processHash) {
        return (execution) => {
            if (!execution.processHash)
                return;
            if (b58.encode(execution.processHash) !== b58.encode(processHash))
                return;
            this.log(this.formatResult(execution));
        };
    }
    formatResult(execution) {
        if (!execution.instanceHash)
            return;
        const prefix = [
            `[${execution.nodeKey}]`,
            b58.encode(execution.instanceHash),
            this.services[b58.encode(execution.instanceHash)].sid,
            execution.taskKey,
        ].join(' - ');
        if (execution.error) {
            return `${prefix}: ` + chalk_1.default.red('ERROR:', execution.error);
        }
        if (!execution.outputs)
            return;
        return prefix +
            '\n\tinputs:  ' + chalk_1.default.gray(util_1.inspect(encoder_1.decode(execution.inputs || {}))) +
            '\n\toutputs: ' + chalk_1.default.gray(util_1.inspect(encoder_1.decode(execution.outputs || {}))) +
            '\n';
    }
}
exports.default = ProcessLogs;
ProcessLogs.description = 'Log the executions related to a process';
ProcessLogs.flags = Object.assign({}, root_command_1.default.flags);
ProcessLogs.args = [{
        name: 'PROCESS_HASH',
        required: true,
    }];
