"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const compiler_1 = require("@mesg/compiler");
const path_1 = require("path");
const root_command_1 = __importDefault(require("../../root-command"));
const deployer_1 = __importStar(require("../../utils/deployer"));
const ipfsClient = require('ipfs-http-client');
class ServiceCompile extends root_command_1.default {
    constructor() {
        super(...arguments);
        this.IPFS = ipfsClient('ipfs.app.mesg.com', '5001', { protocol: 'http' });
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { args } = this.parse(ServiceCompile);
            this.spinner.status = 'Download sources';
            const path = yield deployer_1.default(args.SERVICE);
            const definition = yield compiler_1.service(fs_1.readFileSync(path_1.join(path, 'mesg.yml')));
            definition.source = yield this.deploySources(path);
            this.styledJSON(definition);
            this.spinner.stop();
            return definition;
        });
    }
    deploySources(path) {
        return __awaiter(this, void 0, void 0, function* () {
            const buffer = [];
            return new Promise((resolve, reject) => {
                deployer_1.createTar(path_1.join(path))
                    .on('data', (data) => buffer.push(...data))
                    .once('error', reject)
                    .on('end', () => __awaiter(this, void 0, void 0, function* () { return resolve(yield this.upload(Buffer.from(buffer))); }));
            });
        });
    }
    upload(buffer) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.IPFS.add(Buffer.from(buffer), { pin: true });
            if (!res.length) {
                throw new Error('pushing service manifest failed: run service:compile again');
            }
            return res[0].hash;
        });
    }
}
exports.default = ServiceCompile;
ServiceCompile.description = 'Compile a service and upload its source to IPFS server';
ServiceCompile.flags = Object.assign({}, root_command_1.default.flags);
ServiceCompile.args = [{
        name: 'SERVICE',
        description: 'Path or url of a service',
        default: './'
    }];
