"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const fs_1 = require("fs");
const node_docker_api_1 = require("node-docker-api");
const debug = require('debug')('docker');
const root_command_1 = __importDefault(require("./root-command"));
class default_1 extends root_command_1.default {
    constructor() {
        super(...arguments);
        this.docker = new node_docker_api_1.Docker(null);
    }
    listServices(options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.docker.service.list({
                filters: { name: [options.name] }
            });
        });
    }
    waitForEvent(matchFilter) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                const events = (yield this.docker.events());
                const handler = (buffer) => {
                    try {
                        const event = JSON.parse(buffer.toString());
                        if (matchFilter(event)) {
                            events.destroy();
                            return resolve();
                        }
                    }
                    catch (_a) { }
                };
                events.addListener('data', handler);
            }));
        });
    }
    getOrCreateNetwork(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const networks = yield this.docker.network.list({
                filters: { name: [options.name] }
            });
            if (networks.length > 0) {
                return networks[0];
            }
            return this.docker.network.create({
                CheckDuplicate: true,
                Driver: 'overlay',
                Name: options.name,
                labels: {
                    'com.docker.stack.namespace': options.name,
                }
            });
        });
    }
    createEngineService(network, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const image = `mesg/engine:${options.version}`;
            if (!fs_1.existsSync(options.path)) {
                fs_1.mkdirSync(options.path);
            }
            return this.docker.service.create({
                Name: options.name,
                Labels: {
                    'com.docker.stack.namespace': options.name,
                    'com.docker.stack.image': image,
                },
                TaskTemplate: {
                    ContainerSpec: {
                        Image: image,
                        Labels: {
                            'com.docker.stack.namespace': options.name
                        },
                        Env: [
                            `MESG_NAME=${options.name}`,
                        ],
                        Mounts: [{
                                Source: '/var/run/docker.sock',
                                Target: '/var/run/docker.sock',
                                Type: 'bind',
                            }, {
                                Source: options.path,
                                Target: '/root/.mesg',
                                Type: 'bind',
                            }],
                    },
                    Networks: [
                        {
                            Target: network.id,
                            Alias: options.name,
                        },
                    ]
                },
                EndpointSpec: {
                    Ports: [{
                            Protocol: 'tcp',
                            PublishMode: 'ingress',
                            TargetPort: 50052,
                            PublishedPort: options.port,
                        }, {
                            Protocol: 'tcp',
                            PublishMode: 'ingress',
                            TargetPort: 26656,
                            PublishedPort: options.p2pPort,
                        }],
                }
            });
        });
    }
    pull(tag) {
        return __awaiter(this, void 0, void 0, function* () {
            const stream = (yield this.docker.image.create({}, {
                fromImage: 'mesg/engine',
                tag
            }));
            return new Promise((resolve, reject) => stream
                .on('data', (x) => debug(x.toString())) // For some reason we need to listen to the data to have the end event
                .on('error', reject)
                .on('end', resolve));
        });
    }
}
exports.default = default_1;
default_1.flags = Object.assign(Object.assign({}, root_command_1.default.flags), { name: command_1.flags.string({
        description: 'Name of the docker service running the engine',
        required: true,
        default: 'engine'
    }) });
